// CNTEXP.C - Universelles Export-Tool fr CNT-Dateien
// Written by Stefan Pfeiffer, DL1ELY, 941225
// Last modification: 950215
// (C) 1994 by Stefan Pfeiffer, DL1ELY

#include <stdio.h>
#include <string.h>
#include "structs.h"

#define TITLE   "CNT-Exporter v1.01, 950215 by Stefan Pfeiffer, DL1ELY\n"
#define CNT_EXT ".CNT"
#define CSV_EXT ".CSV"
#define SDF_EXT ".SDF"

#define T_SDF 0
#define T_CSV 1

char tabs[9] = {0,8,12,16,32,42,51,60,68};

unsigned f_type = T_SDF;
unsigned char delimiter;
unsigned char srcfile[128];
FILE *srcf;
unsigned char dstfile[128];
FILE *dstf;
unsigned char version;


void disp_help()
{
  printf("Aufruf:\n");
  printf("  CNTEXP [Option1] [Option2] [...] <CNT-File>[.CNT] [ASCII-File][.EXT]\n\n");
  printf("Optionen (knnen auch mit '/' eingeleitet werden):\n");
  printf("  -CSV : Schreibt die ASCII-Datei im CSV-Format. Diese kann z.B. in EXCEL\n");
  printf("         importiert werden (inklusive Spaltenbeschriftung).\n");
  printf("         Als Default wird das SDF-Format (ohne Spaltenbeschriftung, Texte in\n");
  printf("         Anfhrungszeichen eingeschlossen) ausgegeben.\n");
  printf("  -Dx  : Legt das Delimiter-Zeichen fest, also das Trennzeichen zwischen den\n");
  printf("         einzelnen Feldern eines Datensatzes. Default ist ';'. Fr 'x' mu\n");
  printf("         das gewnschte Zeichen angegeben werden.\n");
  printf("  -DT  : Benutzt das ASCII-Zeichen 9 (TAB) als Trennzeichen.\n");
  printf("  -DS  : Benutzt genau ein Leerzeichen als Trennzeichen.\n");
  printf("  -DF  : Benutzt soviele Leerzeichen als Trennzeichen, da die Felder der\n");
  printf("         Datenstze genau untereinander stehen (Tabellenform).\n");
  printf("  -?   : Gibt Beispiele zur Benutzung der Optionen aus.\n\n");
  
  printf("Dieses Programm ist (C) 1994-95 by Stefan Pfeiffer, DL1ELY.\n");
  printf("Es darf nur zusammen mit meinem Kontest-Logging-Programm FiveNine\n");
  printf("weitergegeben werden. Es gelten die Bestimmungen aus LIZENZ.DOC.\n");
};
  
  
void disp_examples()
{
  printf("Beispiele:\n");
  printf("  CNTEXP SEP94                Exportiert die Datei SEP94.CNT in die Datei\n");
  printf("                              SEP94.SDF mit dem Trennzeichen ';'.\n\n");
  printf("  CNTEXP -CSV SEP94           Exportiert die Datei SEP94.CNT in die Datei\n");
  printf("                              SEP94.CSV mit dem Trennzeichen ';' (inklusive\n");
  printf("                              Spaltentitel).\n\n");
  printf("  CNTEXP -D# SEP94 SEP94.TXT  Exportiert die Datei SEP94.CNT in die Datei\n");
  printf("                              SEP94.TXT mit dem Trennzeichen '#'.\n\n");
  printf("  CNTEXP /DF /CSV SEP94       Exportiert die Datei SEP94.CNT in die Datei\n");
  printf("                              SEP94.CSV (In Tabellenform mit Spaltentiteln).\n\n");
  
  printf("Dieses Programm ist (C) 1994 by Stefan Pfeiffer, DL1ELY.\n");
  printf("Es darf nur zusammen mit meinem Kontest-Logging-Programm FiveNine\n");
  printf("weitergegeben werden. Es gelten die Bestimmungen aus LIZENZ.DOC.\n");
};

  
void make_csv_header(FILE *f, unsigned char delim)
{
  char tab[9][20] = {"Nummer","Std","Min","Call","RST geg","RST erh","Locator","Punkte","Richtung"};
  
  unsigned i;
  char s[128];
  char hlp[2];
  
  s[0] = 0;
  for(i=0;i<9;i++)
  {
    strcat(s,tab[i]);
    switch(delim)
    {
      case 'T':
        if(i<8) strcat(s,"\x09");
        break;
      case 'S':
        if(i<8) strcat(s," ");
        break;
      case 'F':
        if(i<8)
          while(strlen(s)<tabs[i+1]) strcat(s," ");
        break;
      default:
        if(i<8) 
        {  
          hlp[0] = delim;
          hlp[1] = 0;
          strcat(s,hlp);
        };
        break;
    };
  };
  fprintf(f,"%s\n",s);
};


void make_delim(char *s, unsigned char delim, unsigned char i)
{
  char hlp[2];
  
  switch(delim)
  {
    case 'T':
      strcat(s,"\x09");
      break;
    case 'S':
      strcat(s," ");
      break;
    case 'F':
      while(strlen(s)<tabs[i+1]) strcat(s," ");
      break;
    default:
       hlp[0] = delim;
       hlp[1] = 0;
       strcat(s,hlp);
       break;
  };
};

void write_sdf_line(FILE *f,unsigned char delim, struct qso q)
{
  char s[128];
  char hlp[20];
  char hlp2[20];
  
  s[0] = 0;
  
  sprintf(hlp,"%4d",q.nr);
  strcat(s,hlp);
  make_delim(s,delim,0);

  sprintf(hlp,"%02d:%02d",q.hour,q.min);
  strcat(s,hlp);
  make_delim(s,delim,2);
  
  sprintf(hlp,"%-14s",q.call);
  strcat(s,hlp);
  make_delim(s,delim,3);
  
  if(q.nr<1000) sprintf(hlp,"%s%03d",q.rst_TX,q.nr);
  else sprintf(hlp,"%s%4d",q.rst_TX,q.nr);
  sprintf(hlp2,"%7s",hlp);
  strcat(s,hlp2);
  make_delim(s,delim,4);
  
  sprintf(hlp,"%7s",q.rst_RX);
  strcat(s,hlp);
  make_delim(s,delim,5);
  
  sprintf(hlp,"%s",q.locator);
  strcat(s,hlp);
  make_delim(s,delim,6);
  
  sprintf(hlp,"%4d",q.points);
  strcat(s,hlp);
  make_delim(s,delim,7);
  
  sprintf(hlp,"%3d",q.direction);
  strcat(s,hlp);
  make_delim(s,delim,8);

  fprintf(f,"%s\n",s);
};

void write_csv_line(FILE *f,unsigned char delim, struct qso q)
{
  char s[128];
  char hlp[20];
  
  s[0] = 0;
  
  sprintf(hlp,"%d",q.nr);
  strcat(s,hlp);
  make_delim(s,delim,0);

  sprintf(hlp,"%02d",q.hour);
  strcat(s,hlp);
  make_delim(s,delim,1);
  
  sprintf(hlp,"%02d",q.min);
  strcat(s,hlp);
  make_delim(s,delim,2);
  
  sprintf(hlp,"%s",q.call);
  strcat(s,hlp);
  make_delim(s,delim,3);
  
  if(q.nr<1000) sprintf(hlp,"%s%03d",q.rst_TX,q.nr);
  else sprintf(hlp,"%s%4d",q.rst_TX,q.nr);
  strcat(s,hlp);
  make_delim(s,delim,4);
  
  sprintf(hlp,"%s",q.rst_RX);
  strcat(s,hlp);
  make_delim(s,delim,5);
  
  sprintf(hlp,"%s",q.locator);
  strcat(s,hlp);
  make_delim(s,delim,6);
  
  sprintf(hlp,"%d",q.points);
  strcat(s,hlp);
  make_delim(s,delim,7);
  
  sprintf(hlp,"%d",q.direction);
  strcat(s,hlp);
  make_delim(s,delim,8);

  fprintf(f,"%s\n",s);
};
                  
int main(int argc, char **argv)
{
  int first_arg = 1;
  int i;
  unsigned char *cp;
  unsigned char sig[4];
  struct qso q;
  unsigned count, read;
  
  printf("\n%s\n",TITLE);
  
  // Erstes Argument suchen, das KEINE Option ist.
  for(;(first_arg<argc)&&(argv[first_arg][0]=='-'||argv[first_arg][0]=='/');first_arg++) ;
  
  // Optionen durchgehen
  for(i=1;i<first_arg;i++)
  {
    cp = argv[i]+1;
    strupr(cp);
    
    if(*cp=='?')
    {
      disp_examples();
      return(1);
    };  
    if(!strcmp(cp,"CSV")) f_type = T_CSV;
  };
  
  delimiter = f_type==T_SDF?' ':';';
  
  // Optionen durchgehen
  for(i=1;i<first_arg;i++)
  {
    cp = argv[i]+1;
    strupr(cp);
    
    if(*cp=='D') delimiter = *(++cp);
  };
  
  if(first_arg>=argc)  // Kein Dateiname angegeben
  {
    disp_help();
    return(1);
  };
  
  // Dateinamen kopieren
  strcpy(srcfile,argv[first_arg]);
  dstfile[0] = 0;
  if(first_arg<argc-1) strcpy(dstfile,argv[++first_arg]);
      
  // Default-Extensions anfgen
  cp = strchr(srcfile,'.');
  if(!cp) strcat(srcfile,CNT_EXT);
  strupr(srcfile);
  
  if(!dstfile[0]) strcpy(dstfile,srcfile);
  cp = strchr(dstfile,'.');
  *cp = 0;
  strcat(dstfile,f_type==T_SDF?SDF_EXT:CSV_EXT);
  
  if((srcf = fopen(srcfile,"rb")) == NULL)
  {
    printf("Datei %s nicht gefunden!\n",srcfile);
    return(2);
  };
  
  fread((void *)sig,1,4,srcf);
  if(strncmp(sig,"CNT",3))
  {
    fclose(srcf);
    printf("Datei %s ist kein CNT-File!\n",srcfile);
    return(3);
  };
  
  version = sig[3]=='1'?1:2;
  fseek(srcf,HEADER_SIZE,SEEK_SET);
  
  if((dstf = fopen(dstfile,"wt")) == NULL)
  {
    fclose(srcf);
    printf("Datei %s kann nicht erzeugt werden!\n",dstfile);
    return(4);
  };
        
  printf("%s (CNT%d) -> %s\n",srcfile,version,dstfile);
  
  if(f_type==T_CSV) make_csv_header(dstf,delimiter);
  
  count = 0;
  if(feof(srcf)) return(0);
  
  do
  {
    if(version==1) fseek(srcf,8,SEEK_CUR);
    read =  fread((void *)&q.hour,1,2,srcf);
    read += fread((void *)&q.min,1,2,srcf);
    read += fread((void *)&q.nr,1,2,srcf);
    read += fread((void *)q.barecall,1,version==1?7:13,srcf);
    read += fread((void *)q.call,1,14,srcf);
    read += fread((void *)q.locator,1,7,srcf);
    read += fread((void *)q.rst_RX,1,8,srcf);
    read += fread((void *)q.rst_TX,1,4,srcf);
    read += fread((void *)&q.points,1,2,srcf);
    read += fread((void *)&q.direction,1,2,srcf);
    if(read==(version==1?50:56))
    {
      if(f_type == T_SDF) write_sdf_line(dstf,delimiter,q);
      else write_csv_line(dstf,delimiter,q);
      if(!((count++)%10)) putch('.');
    };
  } while(!feof(srcf));
  
  putch('\n');
  fclose(srcf);
  fclose(dstf);
  
  return(0);
};
      
