/*
  printlog.c - prints CNT-contest-files
  Written by Stefan Pfeiffer, DL1ELY, 940430
  Last modification: 940509
*/  

/* 
  v1.00, 940430: initial version.
  
  v1.01, 940501: Added printing of the 0 with a stroke through it.

  v1.02, 940509: A FormFeed is send after 55 lines.
                 Added headline on each page.
                 Added display of percentage of printing.
*/
                 
#include "structs.h"
#include <stdio.h>
#include <string.h>

#define PRGNAME "PRINTLOG"
#define VERSION "v1.02"
#define AUTHOR "Stefan Pfeiffer, DL1ELY"
#define DATE "940509"

#define FORMFEED "\x0C"
#define DASHED_ZERO "\x1B\x7E\x01"

unsigned char fname[128];
FILE *cntf;
char *cp;
struct header h;
struct qso q;
unsigned read;
unsigned lines;
unsigned page = 0;
unsigned cnt = 0;

void print_header(void);

void print_header()
{
  unsigned i;

  page++;
  for(i=0;i<(80-strlen(h.desc))/2;i++) fprintf(stdout," ");
  fprintf(stdout,"%s\n",h.desc);
  fprintf(stdout,"  Call: %-14s  Locator: %s",h.owncall,h.ownloc);
  fprintf(stdout,"                           ");
  fprintf(stdout,"Seite: %02d\n\n",page);

  for(i=0;i<71;i++) fprintf(stdout,"\xC4");
  fprintf(stdout,"\n");
  
  fprintf(stdout,"\xB3  Nr  ");
  fprintf(stdout,"\xB3 Zeit  ");
  fprintf(stdout,"\xB3  Call          ");
  fprintf(stdout,"\xB3 RST geg ");
  fprintf(stdout,"\xB3 RST erh ");
  fprintf(stdout,"\xB3 Loc.   ");
  fprintf(stdout,"\xB3 Punkte \xB3");
  fprintf(stdout,"\n");
  
  for(i=0;i<71;i++) fprintf(stdout,"\xC4");
  fprintf(stdout,"\n");
};

int main(int argc, char **argv)
{
  unsigned i;
  
  fprintf(stderr,"%s %s by %s, %s\n",PRGNAME,VERSION,AUTHOR,DATE);
  
  if(argc != 2)
  {
    fprintf(stderr,"\nAufruf:\n");
    fprintf(stderr,"  PRINTLOG <Contestfile>[.CNT]\n");
    fprintf(stderr,"\nDas Log wird ber die Standardausgabe ausgegeben.\n");
    fprintf(stderr,"Zum Ausdruck bitte  >PRN , zum Speichern  >filename  verwenden.\n");
    return(1);
  }
  
  strcpy(fname,argv[1]);
  cp = strchr(fname,'.');
  if(!cp) strcat(fname,".CNT");
  strupr(fname);
  
  if((cntf = fopen(fname,"rb")) == NULL)
  {
    fprintf(stderr,"\nFile %s nicht gefunden!\n",fname);
    return(2);
  }
  
  fread((void *)&h,1,HEADER_SIZE,cntf);
  
  if(strncmp(h.magic,"CNT",3))
  {
    fprintf(stderr,"\nFile %s ist kein CNT-File!\n",fname);
    return(3);
  }
      
  
  fprintf(stdout,DASHED_ZERO);
  fprintf(stdout,"\nLog-Ausdruck: %s\n\n",h.desc);
  fprintf(stdout,"Call: %-15s   Locator: %s\n\n",h.owncall,h.ownloc);
  fprintf(stdout,"Verbindungen: %-4d   Davon ungltig: %-2d\n",h.qso_cnt,h.invalid);
  fprintf(stdout,"ODX: %d km (%s)\n",h.ODX,h.ODX_call);
  fprintf(stdout,"Gearbeitete Laender: %-2d   Unbekannte Prefixe: %-d   gearbeitete Grofelder: %-3d\n\n",
                 h.countries,h.unknown_pref,h.squares);
  fprintf(stdout,"Punkte/QSO: %-4d     Gesamtpunktzahl: %-6d\n",h.average,h.points);
                 
  read = 1;
  lines = 0;
  
  while(read)
  {
    read = fread((void *)&q,1,QSO_SIZE,cntf);
    if(read)
    {
      if(!lines) 
      {
        fprintf(stdout,FORMFEED);
        print_header();
      }  
      lines++;
      fprintf(stdout,"\xB3%4d. ",q.nr);
      fprintf(stdout,"\xB3 %02d:%02d ",q.hour,q.min);
      fprintf(stdout,"\xB3 %-14s ",q.call);
      if(q.nr > 999) fprintf(stdout,"\xB3 %3s%4d ",q.rst_TX,q.nr);
      else fprintf(stdout,"\xB3 %3s %03d ",q.rst_TX,q.nr);
      fprintf(stdout,"\xB3 %7s ",q.rst_RX);
      fprintf(stdout,"\xB3 %6s ",q.locator);
      if(q.points) fprintf(stdout,"\xB3 %6d \xB3",q.points);
      else fprintf(stdout,"\xB3    *** \xB3");
      fprintf(stdout,"\n");
      cnt++;
      fprintf(stderr,"%3d%%\r",(cnt/h.qso_cnt)*100);
      if(lines > 54) lines = 0;
    }
  }  
  
  fprintf(stderr,"\n\n");
  for(i=0;i<71;i++) fprintf(stdout,"\xC4");
  fprintf(stdout,FORMFEED);
  fclose(cntf);
  
  return(0);
}
    
