// WAVE.C - Playing Waves on the PC-Speaker
// Written by Stefan Pfeiffer, DL1ELY, 950105
// Last modification: 950302
// (C) 1995 by Stefan Pfeiffer, DL1ELY


// Die Idee zu diesem Source entstand whrend einer langen Nacht
// im Convers auf DB0EAM. Die Idee stammt von Reinald, DH2MBM.
// Ich habe die Umsetzung in den Code besorgt. Es war ein richtiger
// Trial-and-Error-Proze, aber es hat Spa gemacht ;)

// Benutzung dieses Codes:
// if(playwave("TEST.WAV"))
// {
//    do ;
//    while(!play_end());
//    end_wave();
// };

// Man knnte in der Schleife auch noch irgendwas anderes machen.
// ICH habe mich das bisher nicht getraut,hi.

#include <dos.h>
#include <stdio.h>
#include <alloc.h>
#include <string.h>
#include "wave.h"

// Fr is_windows() und is_os2()
#include "tools.h"

// Tabelle mit Timer-Werten
unsigned tab[256];
// Datenpuffer
unsigned char huge *buf;
unsigned seg;

// Fr Samples mit zu niedriger Rate
signed char doppel = 0, nextb = 1;
// Datenlnge
unsigned long len;
// Alter INT 08h
void interrupt (*oldint)();
// Zhler fr 55ms-Emulation
unsigned counter=0;
// Watchdog gegen Selbstunterbrechung
unsigned char activ=0;
// Timer-Faktor
unsigned faktor;
// Altes Speaker-Register
unsigned char oldreg;

// Player-Interrupt
void interrupt player(void);


unsigned dos_get_free_mem(void)
{
  struct REGPACK r;
  
  r.r_ax = 0x4800;
  r.r_bx = 0xFFFF;
  intr(0x21,&r);
  
  return(r.r_bx);
};


unsigned dos_alloc(unsigned paras)
{
  struct REGPACK r;
  
  r.r_ax = 0x4800;
  r.r_bx = paras;
  intr(0x21,&r);
  
  if(!(r.r_flags&1)) return(r.r_ax); else return(0);
};


void dos_free(unsigned segment)
{
  struct REGPACK r;
  
  r.r_ax = 0x4900;
  r.r_es = segment;
  intr(0x21,&r);
};


// Liest WAV-Datei ein, ldt sie und startet den Player
// Folgendes wird vorausgesetzt, aber NICHT anhand des Headers
// berprft:
// 8-Bit-Sample, Mono, unkomprimiert
unsigned playwave(char *fname)
{
  FILE *f;
  unsigned pos1;
  unsigned rate;
  float wert;
  unsigned flen;
  unsigned read;
  unsigned char huge *p;
  signed long loadlen;
  struct riffchunk chunk;
  struct waveformat_tag tag;
  struct wavedata_tag data;

  // Windows und OS/2 mgen die Timerspielereien nicht.
  if(is_windows()||is_os2()) return(0);
  if((f = fopen(fname,"rb")) == NULL) return(0);

  // Chunk einlesen (IFF-Format)
  fread((void *)&chunk,1,sizeof(chunk),f);

  // WAV-Files haben "RIFF" als Identifikator
  if(strncmp(chunk.id,"RIFF",4))
  {
    fclose(f);
    return(0);
  };

  // Und "WAVE" als Data.
  if(strncmp(chunk.data,"WAVE",4))
  {
    fclose(f);
    return(0);
  };

  // Ersten tag einlesen
  fread((void *)&tag,1,sizeof(tag),f);
  if(strncmp(tag.id,"fmt ",4))
  {
    fclose(f);
    return(0);
  };

  // Samplerate bestimmen
  rate = (unsigned)tag.samples_per_sec&0xFFFF;
  // Bei Raten < 20KHz verdoppeln und jedes Byte doppelt spielen.
  // Warum?? Siehe Player-INT.
  if(rate<20000)
  {
    rate*=2;
    doppel = 1;
  };

  // Datentag einlesen.
  fread((void *)&data,1,sizeof(data),f);
  if(strncmp(data.id,"data",4))
  {
    fclose(f);
    return(0);
  };
  len = data.size;
  
  // In Paragraphen umrechnen (Division durch 16)
  flen = (len+15)>>4;
  // Speicher vom DOS anfordern...
  if(dos_get_free_mem()<flen)
  {
    // Nicht genug RAM frei? Dann Maximal-Wert anfordern.
    flen = dos_get_free_mem();
    len = (unsigned long)flen<<4;
  };
  seg = dos_alloc(flen);

  // Zeiger auf erstes Byte bilden
  buf = MK_FP(seg,0);

  // In der folgenden Schleife wird die WAV-Datei blockweise eingelesen
  // und im RAM abgelegt.
  loadlen = len;
  p = buf;
  //error_log("play_wave()","Start reading");
  while(loadlen>0)
  {
    //error_log("play_wave()","Reading %u byte",loadlen>0xF000?0xF000:loadlen);
    read = fread(p,1,loadlen>0xF000?0xF000:loadlen,f);
    //error_log("play_wave()","Read: %u byte",read);
    loadlen-=read;
    p+=read;
  };
  fclose(f);
  //error_log("play_wave()","File closed");

  // Timer-Faktor berechnen
  faktor = (1193180/rate);

  // So, hier wirds kompliziert...
  // Fr jedes Sample-Byte stehen Faktor/1193180 Sekunden zur Verfgung.
  // Die Samplebytes gehen von 0 bis 255. Jedem Wert der Tabelle
  // wird jetzt x/255stel der Zeit zugeordnet, fr x von 0 bis 255.
  // Tabelle[0] enthlt also 0, Tabelle[255] den Wert von Faktor,
  // alle Werte dazwischen die arithmetischen Mittel (sozusagen
  // quidistant :).
  wert = (float)faktor/256;
  for(pos1=0;pos1<256;pos1++) tab[pos1] = (unsigned)(wert*(float)(pos1+1));

  // Speaker-Register einlesen.
  oldreg = inp(0x61);
  // Maskieren.
  oldreg &= 0xFC;
  // 2 untersten Bits setzen.
  outp(0x61,oldreg|3);

  // INT-Ausfhrung blockieren
  activ = 1;
  // INT installieren.
  oldint = getvect(8);
  setvect(8,player);

  //error_log("play_wave()","Programming timer...");

  // INTs sperren
  __emit__(0xFA);
  // Timer-Baustein programmieren.
  // INT 08 wird jetzt im Takt der Samplerate aufgerufen.
  outp(0x43,0x36);
  outp(0x40,faktor&0x00FF);
  outp(0x40,faktor>>8);
  // INTs freigeben
  __emit__(0xFB);

  // Interrupt starten.
  activ = 0;
//  error_log("play_wave()","ends...");
  return(1);
};

void interrupt player()
{
  unsigned reg;

    // INT schon aktiv? EOI an Int-Controller senden und abbrechen.
    if(activ)
    {
      outp(0x20,0x20);
      return;
    };

    // Watchdog setzen
    activ++;
    // Noch Bytes brig?
    if(len)
    {
      // Bytewert holen
      reg = *buf;
      // zugehrigen Timer-Wert holen
      reg = tab[reg&0xFF];
      // Tabellenwert an den Speaker-Timer ausgeben
      // Programmierung als Monoflop:
      // Fr die programmierte Zeit bleibt der Timer-Ausgang auf 1,
      // fllt dann auf 0 ab.
      outp(0x43,0xB2);
      outp(0x42,reg&0xFF);
      outp(0x42,reg>>8);

      // Speaker-Port auslesen und maskieren
      reg = inp(0x61);
      reg &= 0xFE;
      
      // An Bit 0 wird ein Flankenwechsel von 0 auf 1 gemacht.
      // Dadurch wird der Monoflop "angeworfen".
      // Die Membran des Speakers wird jetzt verschieden weit ausgelenkt,
      // je nach Wert des Samplebytes. Dadurch soll die Amplituden-
      // Kurve der Speakerausgabe besser an die Original-Amplitude
      // angepasst werden. Ich htte nie gedacht, da es funktioniert
      // - aber das tut es!
      // Bei niedrigen Sampleraten ist die Membran eventuell schon voll
      // ausgelenkt, obwohl noch Zeit zur Verfgung stnde. Sowas ist
      // mir bisher aber noch nie aufgefallen.
      // ich habe es mit 11KHz und 22KHz-Samples getestet. Vielleicht
      // geht 44KHz auchnoch?? (aufm 486er)
      // Ein 386SX/25 ist fr 22KHz leider zu langsam, auf nem 386DX/40
      // garkein Problem.
      // Weshalb ich aber die Verdopplung der Samplerate mache, falls
      // sie geringer als 20KHz ist:
      // Durch diesen Flankenwechsel entsteht ein Ton von der Frequenz
      // der Samplerate, der bei 8KHz und 11KHz natrlich deutlich
      // hrbar ist, bei 22KHz nicht mehr.
      // Deswegen wird jedes Byte 2mal gespielt in einem solchen Fall.
      // Ich hatte es auch mal mit dem arithmetischen Mittel von 2
      // nebeneinanderliegenden Bytes versucht, das brachte aber
      // komischerweise einen QualittsEINBRUCH!
      outp(0x61,reg);
      outp(0x61,reg|1);
      // Verdoppelte Samplerate?
      if(nextb)
      {
        if(doppel) nextb = 0;
        // 1 Byte mehr gespielt
        len--;
        // Nchstes Byte...
        buf++;
      }
      else nextb = 1;
    };

// Folgendes dient zum Aufrufen des alten INTs 08 im 55ms-Takt.
// Es brachte keinen hrbaren Unterschied, ob dieser Code aktiv war,
// oder nicht. Zur Vermeidung evtl. Schwierigkeiten wird der alte
// INT 08 whrend des Abspielens NICHT aufgerufen. Hat natrlich
// minimale Auswirkungen auf die Systemuhr - nicht zu ndern.

//    if((long)(counter+faktor)>65536)
//    {
//      oldint();
//      counter = (unsigned)((long)(counter+faktor)-65536);
//    }
//    else
//    {
//      counter+=faktor;
      outp(0x20,0x20);
//    };
  activ--;

  return;
};


void end_wave()
{
  // Timerbaustein resetten
  __emit__(0xFA);
  outp(0x43,0x36);
  outp(0x40,0x00);
  outp(0x40,0x00);
  setvect(8,oldint);
  __emit__(0xFB);
  // DOS-Speicher freigeben und Lautsprecher "stummschalten"
  dos_free(seg);
  outp(0x61,oldreg);
};

// Liefert TRUE, wenn alles gespielt wurde.
unsigned play_end()
{
   return(len==0);
};

